﻿using System;
using System.IO;
using System.Runtime.Serialization.Json;
using System.Text;

namespace CallbackForFleetTracker {
    /// <summary>
    /// This class helps to serialize object to Json string and deserialize Json string to Object.
    /// </summary>
    public static class JsonSerializeHelper {
        /// <summary>
        /// This method serialzies an object to Json string.
        /// </summary>
        /// <param name="targetValue">An object value which uses for serialization.</param>
        /// <returns>A string value indicates the serialized Json string</returns>
        public static string Serialize<T>(T targetValue) {
            string result = String.Empty;

            DataContractJsonSerializer jsonSerializer = new DataContractJsonSerializer(targetValue.GetType());
            using (MemoryStream jsonStream = new MemoryStream()) {
                jsonSerializer.WriteObject(jsonStream, targetValue);
                result = Encoding.Default.GetString(jsonStream.GetBuffer());
                jsonStream.Close();
            }
            return result.Trim('\0');
        }

        /// <summary>
        /// This method deserialized a Json string back to the specified object.
        /// </summary>
        /// <param name="targetValue">A Json string which is used for deserialization.</param>
        /// <param name="targetType">Target type of current deserialization.</param>
        /// <returns>An object which is deserialized from the Json string.</returns>
        public static T Deserialize<T>(string targetValue) {
            T obj = Activator.CreateInstance<T>();
            DataContractJsonSerializer jsonSerializer = new DataContractJsonSerializer(obj.GetType());
            byte[] buffer = Encoding.Default.GetBytes(targetValue);
            using (MemoryStream stream = new MemoryStream(buffer)) {
                T result = (T)jsonSerializer.ReadObject(stream);
                return result;
            }
        }
    }
}
